﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;   // Win32 API を使うため


namespace clipboard
{
    public partial class Form1 : Form
    {

        private const int CF_ENHMETAFILE = 14;

        [DllImport("user32.dll")]
        private static extern bool OpenClipboard(IntPtr hWndNewOwner);
        [DllImport("user32.dll")]
        private static extern int IsClipboardFormatAvailable(int wFormat);
        [DllImport("user32.dll")]
        private static extern IntPtr GetClipboardData(int wFormat);
        [DllImport("user32.dll")]
        private static extern int CloseClipboard();

        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            int i, n;
            string line, NL;
            Image img;
            NL = "\r\n";

            textBox2.Text = "";
            textBox3.Text = "";
            textBox4.Text = "";
            pictureBox1.Image = null;
            pictureBox2.Image = null;

            DateTime time = DateTime.Now;
            line = time.ToString("===== yyyy/MM/dd    HH:mm:ss =====") + NL;
            textBox1.AppendText(line);
            
            IDataObject data = Clipboard.GetDataObject();
            if ( data != null)
            {
                // フォーマットの列挙
                int num = data.GetFormats().Length;
                line = string.Format("{0} 個のデータがあります。" + NL, num);
                textBox1.AppendText(line);
                i = 1;
                foreach(string fmt in data.GetFormats())
                {
                    line = string.Format("{0}: {1}" + NL, i, fmt);
                    textBox1.AppendText(line);
                    i++;
                }
                textBox1.AppendText(NL);

                // 画像
                if (Clipboard.ContainsImage())
                {
                    img = Clipboard.GetImage();
                    line = string.Format("画像：幅 {0} px　　高さ {1} px" + NL,
                        img.Width, img.Height);
                    textBox1.AppendText(line);
                    pictureBox2.Image = img;

                }

                // テキスト
                if (Clipboard.ContainsText(TextDataFormat.Text))
                {
                    textBox1.AppendText("テキストがあります" + NL);
                    string text = Clipboard.GetText(TextDataFormat.Text);
                    textBox2.Text = text;
                }

                // ファイル名
                if (Clipboard.ContainsFileDropList())
                {
                    System.Collections.Specialized.StringCollection droplist;
                    droplist = Clipboard.GetFileDropList();
                    n = droplist.Count;
                    line = string.Format("ファイル名が {0} 個あります" + NL, n);
                    textBox1.AppendText(line);
                    string[] stringArray = new string[n];
                    droplist.CopyTo(stringArray, 0);
                    for(i = 0; i < droplist.Count; i++)
                    {
                        textBox3.AppendText(stringArray[i] + NL);
                    }
                    data = Clipboard.GetDataObject();
                    MemoryStream dropeffectStream = data.GetData("Preferred DropEffect") as MemoryStream;
                    if (dropeffectStream != null)
                    {
                        byte[] dropEffectData = new byte[4];
                        dropeffectStream.Read(dropEffectData, 0, dropEffectData.Length);
                        int dropEffectType = BitConverter.ToInt32(dropEffectData, 0);
                        if (dropEffectType == 2) // move
                        {
                            textBox4.Text = "move";
                        } else if ( dropEffectType == 5)
                        {
                            textBox4.Text = "copy";
                        }
                    }
                }

                // メタファイル
                if (Clipboard.ContainsData(DataFormats.EnhancedMetafile))
                {
/*                    Metafile emf = (Metafile)data.GetData(DataFormats.EnhancedMetafile);
                    if (emf == null)
                    {
                        textBox1.AppendText("emf null" + NL);
                    }
*/
                    // NET Framework の Clipboard では EnhancedMetafile を取得できない
                    // https://krdlab.hatenablog.com/entry/20070311/1173603960

                    // Win32 API を使う必要がある
                    // http://dobon.net/vb/dotnet/graphics/getclipboardmetafile.html

                    OpenClipboard(this.Handle);
                    IntPtr hmeta = GetClipboardData(CF_ENHMETAFILE);
                    Metafile emf2 = new System.Drawing.Imaging.Metafile(hmeta, true);
                    CloseClipboard();

                    line = string.Format("拡張メタファイル：幅 {0}　高さ {1}" + NL, emf2.Width, emf2.Height);
                    textBox1.AppendText(line);
                    pictureBox1.Image = emf2;
                }

            }
            textBox1.AppendText(NL);
        }

        private void label2_Click(object sender, EventArgs e)
        {

        }

        private void label3_Click(object sender, EventArgs e)
        {

        }

        private void button3_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            Clipboard.Clear();
        }

        private void pictureBox2_Click(object sender, EventArgs e)
        {

        }
    }
}
